-----------------------------------------------------------------------------
--
-----------------------------------------------------------------------------
function CF_ParseLine(s)
	local pos1 , pos2;
	
	pos = string.find(s ,"=");
	
	if pos ~= nil then
		local  param , value;

		s = string.gsub(s , "\n" , "");
		param = string.sub(s , 1 , pos - 1);
		value = string.sub(s , pos + 1 , string.len(s));

		return param , value;
	else
		return nil;
	end
end
-----------------------------------------------------------------------------
--
-----------------------------------------------------------------------------
function CF_ReadConfigFile(modulename , filename)
	io = require("io");
	local config = {};
	
	for line in io.lines(".\\"..modulename.."\\CampaignData\\"..filename) do
		local param , value;
		
		param, value = CF_ParseLine(line);
		config[param] = value;
	end
	
	return config;
end
-----------------------------------------------------------------------------
--
-----------------------------------------------------------------------------
function CF_WriteConfigFile(config , modulename , filename)
	io = require("io");

	local file = io.open(".\\"..modulename.."\\CampaignData\\"..filename , "w");
	
	for i,line in pairs(config) do
		file:write(tostring(i).."="..tostring(line).."\n");
	end
	
	file:close();
end
-----------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------
function CF_IsFileExists(modulename , filename)
	io = require("io");
	
	local file = io.open(".\\"..modulename.."\\CampaignData\\"..filename , "r");
	
	if file == nil then
		return false;
	end
	
	file:close();
	return true;
end
-----------------------------------------------------------------------------
--
-----------------------------------------------------------------------------
function CF_GetCharPixelWidth(char)
	local ChrLen = {};
	local n = nil;

	ChrLen["1"] = 4;
	ChrLen[" "] = 3;
	ChrLen["!"] = 3;
	ChrLen[","] = 3;
	ChrLen["."] = 3;

	ChrLen["f"] = 5;
	ChrLen["i"] = 3;
	ChrLen["j"] = 4;
	ChrLen["l"] = 3;
	ChrLen["m"] = 8;
	ChrLen["t"] = 5;
	ChrLen["w"] = 8;

	ChrLen["I"] = 3;
	ChrLen["M"] = 8;
	ChrLen["T"] = 5;
	ChrLen["W"] = 8;

	--print(char);
	
	n = ChrLen[char];
	
	--print (n);
	
	if n == nil then
		n = 6;
	end

	return n;
end
-----------------------------------------------------------------------------
--
-----------------------------------------------------------------------------
function CF_GetStringPixelWidth(str)
	local len = 0;
	for i = 1, #str do
		len = len + CF_GetCharPixelWidth(string.sub(str , i , i));
	end
	return len;
end
-----------------------------------------------------------------------------
--
-----------------------------------------------------------------------------
function CF_UpdateLog(config , newline)
	local recordnumber;
	local maxloglines;
	local logrecpixelwidth;
	local pixelwidth;
	
	logrecpixelwidth = 450;
	
	pixelwidth = CF_GetStringPixelWidth(newline);
	
	if pixelwidth < logrecpixelwidth then
		newline = newline..string.rep(" " , math.ceil((logrecpixelwidth - pixelwidth) / 3));
	end
	
	maxloglines = tonumber(config["MaxLogLines"]);
	
	if maxloglines == nil then
		maxloglines = 16;
		config["MaxLogLines"] = maxloglines;
	end
	
	recordnumber = tonumber(config["LastLogRecordNumber"]);
	
	if recordnumber == nil then
		recordnumber = 0;
	end

	recordnumber = recordnumber + 1;
	config["LastLogRecordNumber"] = recordnumber;
	
	for i = 1, maxloglines - 1 do
		config["LogLine"..i] = config["LogLine"..i + 1];
	end
	config["LogLine"..tostring(maxloglines)] = tostring(recordnumber).."   "..newline;
end
-----------------------------------------------------------------------------
--
-----------------------------------------------------------------------------
function CF_GetPrintableLog(config)
	local maxloglines;
	local s = "";
	
	maxloglines = tonumber(config["MaxLogLines"]);
	
	if maxloglines == nil then
		maxloglines = 16;
		config["MaxLogLines"] = maxloglines;
	end

	for i = 1, maxloglines do
		if config["LogLine"..i] ~= nil then
			s = s .. config["LogLine"..i].."\n";
		else
			s = s .. "\n";
		end
	end
	
	return s;
end

-----------------------------------------------------------------------------
--
-----------------------------------------------------------------------------
function CF_GetPrintableLogPart(config , lastn)
	local maxloglines;
	local s = "";
	
	maxloglines = tonumber(config["MaxLogLines"]);
	
	if maxloglines == nil then
		maxloglines = 16;
		config["MaxLogLines"] = maxloglines;
	end

	if lastn < maxloglines then
		for i = maxloglines - lastn + 1, maxloglines do
			if config["LogLine"..i] ~= nil then
				s = s .. config["LogLine"..i].."\n";
			else
				s = s .. "\n";
			end
		end
		
	end

	return s;
end
-----------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------





