-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function SaveScreen:StartActivity()
	self.MaxSavedGames = 6;
	self.ModuleName = "UnmappedLands.rte";
	self.Elements = {};
	
	self.MidX = SceneMan.Scene.Width / 2;
	self.MidY = SceneMan.Scene.Height / 2;
	self.Mid = Vector(self.MidX , self.MidY);
	
	self.LoadSlots = {};

	self.ControlsEnabled = true;
	self.ControlPressed = 0;
	
	for i = 1, self.MaxSavedGames do
		self.Elements[i] = {};
		if CF_IsFileExists(self.ModuleName , "SAVEGAME"..i..".dat") then
			local config = {};
			
			config = CF_ReadConfigFile(self.ModuleName , "SAVEGAME"..i..".dat");
			
			if config["PlayerFaction"] ~= nil then
			
				if config["PlayerFaction"] == "Ronin" then
					self.Elements[i]["Description"] = "Ronin campaign\n";
					self.Elements[i]["Preset"] = "Ronin";
				elseif config["PlayerFaction"] == "Dummy" then
					self.Elements[i]["Description"] = "Dummy campaign\n";
					self.Elements[i]["Preset"] = "Dummy";
				elseif config["PlayerFaction"] == "Coalition" then
					self.Elements[i]["Description"] = "Coalition campaign\n";
					self.Elements[i]["Preset"] = "Coalition";
				elseif config["PlayerFaction"] == "Bearforce" then
					self.Elements[i]["Description"] = "Bearforce campaign\n";
					self.Elements[i]["Preset"] = "Bearforce";
				end

				self.Elements[i]["Description"] = self.Elements[i]["Description"] ..
					"Turn #"..config["TurnNumber"].." , "..config["PlayerGold"].." oz of gold. "..
					config["Deaths"].." dead vs "..config["Kills"].." killed.";

				self.LoadSlots[i] = true;
			else
				self.Elements[i]["Description"] = "Empty\Broken slot #"..i.."\n";
				self.Elements[i]["Preset"] = "Empty";
				
				self.LoadSlots[i] = false;
			end
		else
			self.Elements[i]["Description"] = "Empty slot #"..i.."\n";
			self.Elements[i]["Preset"] = "Empty";

			self.LoadSlots[i] = false;
		end
	end
	
	self.Elements[1]["NextPlate"] = {6 , 2 , 7 , 7}; -- {L , R , U , D}
	self.Elements[2]["NextPlate"] = {1 , 3 , 7 , 7}; -- {L , R , U , D}
	self.Elements[3]["NextPlate"] = {2 , 4 , 7 , 7}; -- {L , R , U , D}
	self.Elements[4]["NextPlate"] = {3 , 5 , 7 , 7}; -- {L , R , U , D}
	self.Elements[5]["NextPlate"] = {4 , 6 , 7 , 7}; -- {L , R , U , D}
	self.Elements[6]["NextPlate"] = {5 , 1 , 7 , 7}; -- {L , R , U , D}
	
	self.Elements[self.MaxSavedGames + 1] = {};
	self.Elements[self.MaxSavedGames + 1]["Description"] = "Exit to strategy screen";
	self.Elements[self.MaxSavedGames + 1]["Preset"] = "ButtonExit";
	self.Elements[self.MaxSavedGames + 1]["NextPlate"] = {7 , 7 , 3 , 3}; -- {L , R , U , D}

	self.Selection = 1;
	if CF_IsFileExists(self.ModuleName , "CURRENT.dat") then
		local config = {};
			
		config = CF_ReadConfigFile(self.ModuleName , "CURRENT.dat");
		
		self.Selection = tonumber(config["DefaultSlot"]);
	end
	
	self.XTiles = 6; -- Plates per row
	
	self.HelpText = "Use LEFT, RIGHT, UP, DOWN to chose SAVE slot, press FIRE to select\n";

	self.ElementsPos = {};
	
	for i = 1 , self.MaxSavedGames do
		self.ElementsPos[i] = Vector(30 + (self.MidX) - (self.XTiles * 62 / 2) + ((i - 1) % self.XTiles) * 62  , 
									    self.MidY);
	end

	--self.ElementsPos[self.MaxSavedGames + 1] = self.Mid + Vector(130 , 41);	
	--self.ElementsPos[self.MaxSavedGames + 2] = self.Mid + Vector(139 , 61);

	self.ElementsPos[self.MaxSavedGames + 1] = self.Mid + Vector(0 , 48);	

	
	self.Plates = {};
	
	--Make an invisible brain.
	self.brain = CreateActor("Brain Case");
	self.brain.Scale = 0;
	self.brain.Team = Activity.TEAM_1;
	self.brain.Pos = self.Mid;
	MovableMan:AddActor(self.brain);
	self:SetPlayerBrain(self.brain,Activity.TEAM_1);
	self:SwitchToActor(self.brain,Activity.PLAYER_1,-1);

	self.SlotBack = CreateMOSRotating("SlotBackground" , self.ModuleName);
	self.SlotBack.Pos = self.Mid + Vector(0 , 0);
	MovableMan:AddParticle(self.SlotBack);	

	self.Title = CreateMOSRotating("TitleSaveGame" , self.ModuleName);
	self.Title.Pos = self.Mid + Vector(0, -70);
	MovableMan:AddParticle(self.Title);	
	
	self.Selection = 1;
	self:RedrawAllPlates();

	self.GenericTimer = Timer();
	self.GenericTimer:Reset();
	
	self.UITimer = Timer();
	self.UITimer:Reset();
	
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
-- This function is called when mission is paused
function SaveScreen:PauseActivity(pause)
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
-- This function is called after mission has ended
function SaveScreen:EndActivity()
end

-----------------------------------------------------------------------------------------
-- Redraw load menu plate
-----------------------------------------------------------------------------------------
function SaveScreen:RedrawPlate(platenum , isselected , ispressed)
	if MovableMan:IsParticle(self.Plates[platenum]) then
		MovableMan:RemoveParticle(self.Plates[platenum]);
		self.Plates[platenum] = nil;
	end

	local preset;

	if platenum >= 1 and platenum <= 6 then
		if self.Elements[platenum]["Preset"] == "Empty"  then
			if isselected then
				preset = "Slot"..platenum.."EmptySaveTo";
			else
				preset = "Slot"..platenum.."Empty";
			end
			
			if ispressed then
				preset = "Slot"..platenum.."EmptyPressed";
			end
		else
			if isselected then
				preset = "Slot"..platenum..self.Elements[platenum]["Preset"].."Overwrite";
			else
				preset = "Slot"..platenum..self.Elements[platenum]["Preset"].."Idle";
			end

			if ispressed then
				preset = "Slot"..platenum..self.Elements[platenum]["Preset"].."OverwritePressed";
			end
		end
	else
		if isselected then
			preset = self.Elements[platenum]["Preset"].."MouseOver";
		else
			preset = self.Elements[platenum]["Preset"].."Idle";
		end
		
		if ispressed then
			preset = self.Elements[platenum]["Preset"].."Pressed";
		end
	end

	self.Plates[platenum] = CreateMOSRotating(preset , self.ModuleName);
	if isselected then
		self.Plates[platenum].SpriteAnimMode = 1;
	end
	
	self.Plates[platenum].Pos = self.ElementsPos[platenum];
	MovableMan:AddParticle(self.Plates[platenum]);	
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function SaveScreen:RedrawAllPlates()
	for i = 1, #self.Elements do
		self:RedrawPlate(i , false , false);
	end

	self:RedrawPlate(self.Selection , true , false);
end

-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function SaveScreen:SaveGame(slot)
	local config;

	config = CF_ReadConfigFile(self.ModuleName , "CURRENT.dat");
	
	config["DefaultSlot"] = slot;
	
	CF_WriteConfigFile(config , self.ModuleName , "SAVEGAME"..slot..".dat");
	
	ActivityMan:EndActivity();
	ActivityMan:StartActivity("GAScripted" , "StrategyScreen");
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function SaveScreen:UpdateActivity()
	--Move the cursor.
	local cont = self.brain:GetController();

	FrameMan:SetScreenText(self.HelpText..self.Elements[self.Selection]["Description"], Activity.PLAYER_1, 0, 1, false);
	-- Add delay to avoid triggering multiple button hits when player holds key for too long
	if self.GenericTimer:IsPastSimMS(350) then
		if self.ControlsEnabled then
			if cont:IsState(Controller.PRESS_LEFT) then
				self:RedrawPlate(self.Selection , false , false);
				self.Selection = self.Elements[self.Selection]["NextPlate"][1];
				self:RedrawPlate(self.Selection , true , false);
			end

			if cont:IsState(Controller.PRESS_RIGHT) then
				self:RedrawPlate(self.Selection , false , false);
				self.Selection = self.Elements[self.Selection]["NextPlate"][2];
				self:RedrawPlate(self.Selection , true , false);
			end

			if cont:IsState(Controller.PRESS_UP) then
				self:RedrawPlate(self.Selection , false , false);
				self.Selection = self.Elements[self.Selection]["NextPlate"][3];
				self:RedrawPlate(self.Selection , true , false);
			end

			if cont:IsState(Controller.PRESS_DOWN) then
				self:RedrawPlate(self.Selection , false , false);
				self.Selection = self.Elements[self.Selection]["NextPlate"][4];
				self:RedrawPlate(self.Selection , true , false);
			end
			

			--If fire has been pushed, start mission.
			if cont:IsState(Controller.WEAPON_FIRE) then
				print (self.Selection);
				self.ControlPressed = self.Selection;
				self.ControlsEnabled = false;
				self.UITimer:Reset();
				self:RedrawPlate(self.Selection , false , true);
				self.GenericTimer:Reset();
			end
		end
	end

	if self.ControlPressed > 0 then
		if self.UITimer:IsPastSimMS(350) then
			self:RedrawPlate(self.ControlPressed , true , false);

			if self.Selection >= 1 and self.Selection <= self.MaxSavedGames then
				self:SaveGame(self.Selection);
				ActivityMan:EndActivity();
				ActivityMan:StartActivity("GAScripted" , "StrategyScreen");
			end

			if self.Selection == 7 then
				ActivityMan:EndActivity();
				ActivityMan:StartActivity("GAScripted" , "StrategyScreen");
			end
			
			self.ControlsEnabled = true;
			self.ControlPressed = 0;				
		end
	end
end
-----------------------------------------------------------------------------------------
-- Thats all folks!!!
-----------------------------------------------------------------------------------------