-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function Attack00:StartActivity()
	print("START! -- Attack00:StartActivity()!");

	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	self.ConfigName = "Attack00.dat";
	self.ModuleName = "UnmappedLands.rte";
	
	self.Config = CF_ReadConfigFile(self.ModuleName , self.ConfigName);
	CF_InitFactions();
	CF_InitMission(self.Config);
	-- end of generic part	
	---------------------------------------------------------------------
	
	self.braindead = {};
	self.AssignedBrains = {};

	self:ObjectiveAssassinateInit();
	self:ObjectiveEvacuateInit();
	
	for player = 0, self.PlayerCount - 1 do
        if not self:GetPlayerBrain(player) then
			self.braindead[player] = false;
			local brain = CF_SpawnBrain(CF_PlayerTeam , nil , CF_PlayerFaction , Actor.AIMODE_SENTRY);
			brain.Pos = Vector(150 + player * 30 , 550);
			brain.AIMode = Actor.AIMODE_SENTRY;
			self.AssignedBrains[player] = brain;

            MovableMan:AddActor(brain);
            self:SetPlayerBrain(brain, player);
            SceneMan:SetScroll(self:GetPlayerBrain(player).Pos, player);
            self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player);
			self:SwitchToActor(brain, player, CF_PlayerTeam);
        end
    end

	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	CF_ReplaceAddedActors(self.Config);
	CF_ArrangeGarrisonActors(self.Config);
	-- end of generic part	
	---------------------------------------------------------------------
	
	self.GenericTimer = Timer();
	self.GenericTimer:Reset();

	self.DropshipSpawnTimer = Timer();
	self.DropshipSpawnInterval = 8000;
	self.DropshipSpawnTimer:Reset();
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
function Attack00:PauseActivity(pause)
    print("PAUSE! -- Attack00:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
function Attack00:EndActivity()
    print("END! -- Attack00:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- Spawn dropship. Returns true if spawn is successfull, false otherwise
-----------------------------------------------------------------------------------------
function Attack00:SpawnDropship()
	if CF_AIBudget <= 0 then
		local enemyactors;
		
		enemyactors = 0;
		
		for actor in MovableMan.Actors do
			if actor.Team == CF_CPUTeam then
				enemyactors = enemyactors + 1;
				
				self:AddObjectivePoint("Kill", actor.AboveHUDPos, CF_PlayerTeam, GameActivity.ARROWDOWN);
			end
		end

		if CF_Activity.ActivityState ~= Activity.OVER then
			if enemyactors <= 1 then
				for player = 0, self.PlayerCount - 1 do
					self:ResetMessageTimer(player);
					FrameMan:ClearScreenText(player);
					FrameMan:SetScreenText("ENEMY RETREATED! VICTORY!!!", player, 1000, 8000, true);
				end

				---------------------------------------------------------------------
				-- Mission framework generic part. Must present in every mission!
				CF_CommitVictory(self.Config , self.ModuleName , self.ConfigName);
				-- end of generic part
				---------------------------------------------------------------------
				self.WinnerTeam = CF_PlayerTeam;
				ActivityMan:EndActivity();
			end
		end
	else
		if self.DropshipSpawnTimer:IsPastSimMS(self.DropshipSpawnInterval) then
			if MovableMan:GetMOIDCount() < CF_MOIDLimit - 20 then
				local ship = nil;
				local actor = nil;
				local price = 0;
				local totalprice = 0;
				
				ship , price = CF_SpawnDropship(CF_CPUTeam , Vector(2000 , 0) , CF_CPUFaction);
				totalprice = totalprice + price;
				
				for i = 1, 2 do
					actor , price = CF_SpawnRandomInfantry(CF_CPUTeam , nil , CF_CPUFaction , Actor.AIMODE_BRAINHUNT);
					totalprice = totalprice + price;

					ship:AddInventoryItem(actor);
				end
				
				CF_AIBudget = CF_AIBudget - totalprice;
				print ("Spawn price "..totalprice);
				print ("AI budget "..CF_AIBudget);
				
				if CF_AIBudget < 0 then
					for player = 0, self.PlayerCount - 1 do
						self:ResetMessageTimer(player);
						FrameMan:ClearScreenText(player);
						FrameMan:SetScreenText("ENEMY RETREATING!\nELEMINATE REMAINING FORCES!", player, 500, 10000, true);
					end
				end

				MovableMan:AddActor(ship);
				self.DropshipSpawnTimer:Reset();
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function Attack00:UpdateActivity()
   	self:ClearObjectivePoints();
	
	self:SpawnDropship();

	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	CF_ReturnOnMissionEnd();
	-- end of generic part
	---------------------------------------------------------------------
	
	
	self:ObjectiveAssassinateUpdate();
	self:ObjectiveEvacuateUpdate();
	
	self:DoBrainSelection();
	
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function Attack00:DoBrainSelection()
	if not (self.ActivityState == Activity.OVER) then
		for player = 0, self.PlayerCount - 1 do
			local team = self:GetTeamOfPlayer(player);
	
			local brain = self:GetPlayerBrain(player);
			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				self:SetPlayerBrain(nil, player);
				local newBrain = MovableMan:GetUnassignedBrain(team);
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self:SwitchToActor(newBrain, player, team);
				else
					self.braindead[player] = true;
					
					if not MovableMan:GetClosestBrainActor(team) then
						self.WinnerTeam = self:OtherTeam(team);
						ActivityMan:EndActivity();
					end
				end
			else
				self:SetObservationTarget(brain.Pos, player)
			end -- 
		end --for
	end --if
end
-----------------------------------------------------------------------------------------
-- Mission objective "Assassinate" initialization
-----------------------------------------------------------------------------------------
function Attack00:ObjectiveAssassinateInit()
	print ("Assasinate Init");
	self.AssassinateComplete = false;
	
	self.Target = CreateAHuman("Dafred");
	self.Target.Pos = Vector(2500 , 559);
	self.Target.Team = CF_CPUTeam;
	self.Target.AIMode = Actor.AIMODE_SENTRY;
	MovableMan:AddActor(self.Target);
end
-----------------------------------------------------------------------------------------
-- Mission objective "Assassinate" update state
-----------------------------------------------------------------------------------------
function Attack00:ObjectiveAssassinateUpdate()
	if self.AssassinateComplete == false then
		if MovableMan:IsActor(self.Target) then
			self:AddObjectivePoint("Assassinate", self.Target.AboveHUDPos, CF_PlayerTeam, GameActivity.ARROWDOWN);
		else
			print ("Assasinate complete");
			self.AssassinateComplete = true;
			self.Target = nil;
		end
	end --if 
end -- func
-----------------------------------------------------------------------------------------
-- Mission objective "Evacuate" initialization
-----------------------------------------------------------------------------------------
function Attack00:ObjectiveEvacuateInit()
	self.EvacuateComplete = false;
end
-----------------------------------------------------------------------------------------
-- Mission objective "Evacuate" update state
-----------------------------------------------------------------------------------------
function Attack00:ObjectiveEvacuateUpdate()
	local wehaverocket = false;
	
	-- Check if we have a rocket to evacuate on. Give player a new one if not. You'll need to check for other
	-- ship's preset names in more complex missions with dropships and other ships.
	if self.AssassinateComplete and not self.EvacuateComplete then
		for actor in MovableMan.Actors do
			if actor.PresetName == "Rocket MK2" and actor.Team == CF_PlayerTeam then
				wehaverocket = true;
				self:AddObjectivePoint("Evacuate", actor.AboveHUDPos, CF_PlayerTeam, GameActivity.ARROWUP);
			end --if
		end -- for	
	
		if not wehaverocket then
			self:ObjectiveEvacuateNewRocket();
		end --if
	end --if
end -- func
-----------------------------------------------------------------------------------------
-- Mission objective "Evacuate" spawn new rocket
-----------------------------------------------------------------------------------------
function Attack00:ObjectiveEvacuateNewRocket()
	print ("New Rocket");

	local rocket = CreateACRocket("Rocket MK2" , "Base.rte");
	rocket.Pos = Vector(800 , -10);
	rocket.Team = CF_PlayerTeam;
	rocket:SetControllerMode(Controller.CIM_AI, -1);

	MovableMan:AddActor(rocket);
end
-----------------------------------------------------------------------------------------
-- Enter orbit, check winning conditions
-----------------------------------------------------------------------------------------
function Attack00:CraftEnteredOrbit()
	if self.OrbitedCraft.Team == CF_PlayerTeam then
		if self.AssassinateComplete == true then
			--If a craft leaves with the brain robot...
			if self.OrbitedCraft:HasObject("Brain Robot") then
				if CF_Activity.ActivityState ~= Activity.OVER then
					-- Team 1 wins!
					self.EvacuateComplete = true;
					-- Tell the script to stop running for brains as the game has ended.
					for player = 0, self.PlayerCount - 1 do
						self.braindead[player] = true;
					end
					
					---------------------------------------------------------------------
					-- Mission framework generic part. Must present in every mission!
					CF_CommitVictory(self.Config , self.ModuleName , self.ConfigName);
					-- end of generic part
					---------------------------------------------------------------------
					self.WinnerTeam = CF_PlayerTeam;
					ActivityMan:EndActivity();
				end
			end
		else
			print (CF_BrainPresets[CF_PlayerFaction]);
			if self.OrbitedCraft:HasObject(CF_BrainPresets[CF_PlayerFaction]) then
				---------------------------------------------------------------------
				-- Mission framework generic part. Must present in every mission!
				CF_CommitRetreat(self.Config , self.ModuleName , self.ConfigName);
				-- end of generic part
				---------------------------------------------------------------------
				self.WinnerTeam = CF_CPUTeam;
				ActivityMan:EndActivity();
			end
		end
	else
		if self.OrbitedCraft.Team == CF_CPUTeam then
			CF_AIBudget = CF_AIBudget + (self.OrbitedCraft.Health / 100 * CF_BasicDropshipPrice);
			print ("Dropship sold CF_AIBudget = " .. CF_AIBudget);
		end
	end
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------



































