-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function Attack01:StartActivity()
	print("START! -- Attack01:StartActivity()!");

	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	-- self.ConfigName = "Attack01.dat";
	self.ConfigName = "current.dat";
	self.ModuleName = "UnmappedLands.rte";
	
	self.Config = CF_ReadConfigFile(self.ModuleName , self.ConfigName);
	CF_InitFactions();
	CF_InitMission(self.Config);
	-- end of generic part	
	---------------------------------------------------------------------

	self.EnemyLZ = SceneMan.Scene:GetArea("EnemyLZ");
	self.Base = SceneMan.Scene:GetArea("Base");
	self.DownloadArea = SceneMan.Scene:GetArea("DownloadArea");
	self.DistributionPoint = SceneMan.Scene:GetArea("DistributionPoint");
	
	self.Posts = {};
	for i = 1 , 5 do
		self.Posts[i] = SceneMan.Scene:GetArea("Post"..i);
	end

	self.IsPostEmpty = {};
	for i = 1 , 5 do
		self.IsPostEmpty[i] = true;
	end
	
	self.BasePosts = {};
	for i = 1 , 5 do
		self.BasePosts[i] = SceneMan.Scene:GetArea("BasePost"..i);
	end
	
	self.IsBasePostEmpty = {};
	for i = 1 , 5 do
		self.IsBasePostEmpty[i] = true;
	end
	
	self.IsBaseEmpty = true;
	

	self.ChunksComplete = 0;
	self.TotalChunks = 100;
	self.ChunkInterval = 700;
	
	self.ScreenActivated = false;
	
	self.braindead = {};
	self.AssignedBrains = {};
	
	for player = 0, self.PlayerCount - 1 do
        if not self:GetPlayerBrain(player) then
			self.braindead[player] = false;
			local brain = CF_SpawnBrain(CF_PlayerTeam , nil , CF_PlayerFaction , Actor.AIMODE_SENTRY);
			-- brain.Pos = Vector(192 , 264); -- Debug
			brain.Pos = Vector(4330 + player * 30 , 380); -- Gameplay
			brain.AIMode = Actor.AIMODE_SENTRY;
			self.AssignedBrains[player] = brain;

            MovableMan:AddActor(brain);
            self:SetPlayerBrain(brain, player);
            SceneMan:SetScroll(self:GetPlayerBrain(player).Pos, player);
            self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player);
			self:SwitchToActor(brain, player, CF_PlayerTeam);
        end
    end

	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	CF_ReplaceAddedActors(self.Config);
	CF_ArrangeEliteActors(self.Config);
	-- end of generic part	
	---------------------------------------------------------------------
	
	self:SpawnBazookaActors();

    self.FightPhase = { NOFIGHT = 0 , DOWNLOAD = 1 , EVACUATE = 2};
    self.CurrentFightPhase = self.FightPhase.NOFIGHT;
	
	self.GenericTimer = Timer();
	self.GenericTimer:Reset();

	self.DropshipSpawnInterval = 20000;
	self.DropshipSpawnTimer = Timer();
	self.DropshipSpawnTimer:Reset();
	
	for actor in MovableMan.AddedActors do
		actor.AIMode = Actor.AIMODE_SENTRY;
	end	

	-- Debug
	--[[self.Screen = CreateAEmitter("Att01 Download Screen");
	self.Screen.Pos = Vector(192 , 264);
	self.Screen:EnableEmission(true);
	MovableMan:AddParticle(self.Screen); --]]--
	
	
	--[[for actor in MovableMan.AddedActors do
		if actor.Team == CF_CPUTeam then
			actor:GibThis();
		end
	end
	
	--]]--
	
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
function Attack01:PauseActivity(pause)
    print("PAUSE! -- Attack01:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
function Attack01:EndActivity()
    print("END! -- Attack01:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function Attack01:SpawnBazookaActors()
	for i = 1 , #self.Posts do
		local newactor, price;

		-- Create unarmed infantry unit
		newactor , price = CF_SpawnInfantry(false , false , false , false , CF_CPUTeam , nil , CF_CPUFaction , Actor.AIMODE_SENTRY);
		newactor.Pos = self.Posts[i]:GetCenterPoint();
	
		if CF_CPUFaction == CF_Factions.COALITION then
			newactor:AddInventoryItem(CreateHDFirearm("Homing Missile Launcher" , "Coalition.rte"));
		elseif CF_CPUFaction == CF_Factions.DUMMY then
			newactor:AddInventoryItem(CreateHDFirearm("Dummy APR Launcher" , self.ModuleName));
		elseif CF_CPUFaction == CF_Factions.RONIN then
			newactor:AddInventoryItem(CreateHDFirearm("Bazooka" , "Ronin.rte"));
		end
		
		MovableMan:AddActor(newactor);
    end
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function Attack01:DistributeReinforcements()
	-- Assign falling troops to specific firing points
	for actor in MovableMan.Actors do
		if actor.Team == CF_CPUTeam and actor.AIMode == Actor.AIMODE_SENTRY then
			if self.DistributionPoint:IsInside(actor.Pos) then
				local isassigned;
				isassigned = false;
				for i = 1 , #self.IsBasePostEmpty do
					if self.IsBasePostEmpty[i] then
						actor.AIMode = Actor.AIMODE_GOTO;
						actor:ClearAIWaypoints();
						actor:AddAISceneWaypoint(self.BasePosts[i]:GetCenterPoint());
						
						isassigned = true;
						break;
					end -- if
				end -- for i
				
				-- If all points in selected zone are taken, assign to brain hunt
				-- to avoid crowds of sentry troops
				if not isassigned then
					actor.AIMode = Actor.AIMODE_BRAINHUNT;
				end
			end -- if 
		end -- if
	end
end
-----------------------------------------------------------------------------------------
-- Update base defence stats to make dispatch decisions later
-----------------------------------------------------------------------------------------
function Attack01:UpdateBaseDefenceStats()
	-- Reset all current states, they are refilled here
	-- Zones are considered non-empty if we have non-expired troops sent to this zone
	for i = 1 , #self.IsPostEmpty do
		self.IsPostEmpty[i] = true;
	end
	
	for i = 1 , #self.IsBasePostEmpty do
		self.IsBasePostEmpty[i] = true;
	end
	
	self.IsBaseEmpty = true;
	
	for actor in MovableMan.Actors do
		-- Check for empty firing points
		if actor.Team == CF_CPUTeam  then
			for i = 1 , #self.Posts do
				if self.Posts[i]:IsInside(actor.Pos) then
					self.IsPostEmpty[i] = false;
				end -- if
			end -- for

			for i = 1 , #self.BasePosts do
				if self.BasePosts[i]:IsInside(actor.Pos) then
					self.IsBasePostEmpty[i] = false;
				end -- if
			end -- for
		end -- if

		if actor.Team == CF_PlayerTeam  then
			if self.Base:IsInside(actor.Pos) then
				self.IsBaseEmpty = false;
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- Spawn dropship. Returns true if spawn is successfull, false otherwise
-----------------------------------------------------------------------------------------
function Attack01:SpawnDropship()
	if self.DropshipSpawnTimer:IsPastSimMS(self.DropshipSpawnInterval) then
		if MovableMan:GetMOIDCount() < CF_MOIDLimit - 20 then
			local ship = nil;
			local actor = nil;
			local price = 0;
			local totalprice = 0;
			local p;
			local landpos;
			local spawnaa; -- Flags if we need to spawn Anti Aircraft troops
			
			spawnaa = false;
			
			-- Select landing zone
			landpos = Vector(self.EnemyLZ:GetCenterPoint().X , -10);
			
			-- Add AA troopers if AI base is empty or with 15% probability if it is not
			if self.IsBaseEmpty or math.random() < 0.15 then
				for i = 1, #self.IsPostEmpty do
					if self.IsPostEmpty[i] then
						landpos = Vector(self.Posts[i]:GetCenterPoint().X , -10);
						spawnaa = true;
						break;
					end
				end
			end
			
			ship , price = CF_SpawnDropship(CF_CPUTeam , landpos , CF_CPUFaction);
			totalprice = totalprice + price;
			
			if spawnaa then
				actor , price = CF_SpawnInfantry(false , false , false , false , CF_CPUTeam , nil , CF_CPUFaction , Actor.AIMODE_SENTRY);
			
				if CF_CPUFaction == CF_Factions.COALITION then
					actor:AddInventoryItem(CreateHDFirearm("Homing Missile Launcher" , "Coalition.rte"));
				elseif CF_CPUFaction == CF_Factions.DUMMY then
					actor:AddInventoryItem(CreateHDFirearm("Dummy APR Launcher" , self.ModuleName));
				elseif CF_CPUFaction == CF_Factions.RONIN then
					actor:AddInventoryItem(CreateHDFirearm("Bazooka" , "Ronin.rte"));
				end

				ship:AddInventoryItem(actor);				
			else
				local aimode;
				
				aimode = Actor.AIMODE_SENTRY;
				
				if self.CurrentFightPhase ~= self.FightPhase.NOFIGHT then
					aimode = Actor.AIMODE_BRAINHUNT;
				end
			
				-- Spawn armor or regular troopers depending on turn number to increase difficulty
				if math.random(100) < 10 then
					-- Give single dreadnought as dummy armor or two assalut troopers with shields
					-- to any other faction
					if CF_CPUFaction == CF_Factions.DUMMY then
						actor , price = CF_SpawnArmor(CF_CPUTeam , nil , CF_CPUFaction , Actor.AIMODE_SENTRY);
						totalprice = totalprice + price;
						
						ship:AddInventoryItem(actor);
					else
						actor , price = CF_SpawnArmor(CF_CPUTeam , nil , CF_CPUFaction , Actor.AIMODE_SENTRY);
						totalprice = totalprice + price;

						ship:AddInventoryItem(actor);
					end
				else
					actor , price = CF_SpawnRandomInfantry(CF_CPUTeam , nil , CF_CPUFaction , Actor.AIMODE_SENTRY);
					totalprice = totalprice + price;

					ship:AddInventoryItem(actor);
				end
			end

			MovableMan:AddActor(ship);
		end
		self.DropshipSpawnTimer:Reset();
	end
end
-----------------------------------------------------------------------------------------
-- Phase NOFIGHT
-----------------------------------------------------------------------------------------
function Attack01:PhaseNoFight()
	self:AddObjectivePoint("Download Security Codes", self.DownloadArea:GetCenterPoint() + Vector(0 , -25), CF_PlayerTeam, GameActivity.ARROWDOWN);

	for player = 0, self.PlayerCount - 1 do
		if MovableMan:IsActor(self.AssignedBrains[player]) then
			if not self.GenericTimer:IsPastSimMS(5000) then
				self:AddObjectivePoint("Protect Commander", self.AssignedBrains[player].AboveHUDPos, CF_PlayerTeam, GameActivity.ARROWDOWN);
			end
			
			if self.DownloadArea:IsInside(self.AssignedBrains[player].Pos) then
				self:AdvanceToDownload();
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function Attack01:AdvanceToDownload()
	self.CurrentFightPhase = self.FightPhase.DOWNLOAD;
end
-----------------------------------------------------------------------------------------
-- Phase 
-----------------------------------------------------------------------------------------
function Attack01:PhaseDownload()
	local isinside = false;

	-- Show boot completion mark
	for player = 0, self.PlayerCount - 1 do
		if MovableMan:IsActor(self.AssignedBrains[player]) then
			if self.DownloadArea:IsInside(self.AssignedBrains[player].Pos) then
				isinside = true;
			
				if not self.ScreenActivated then
					self.Screen = CreateAEmitter("Att01 Download Screen");
					self.Screen.Pos = Vector(192 , 264);
					self.Screen:EnableEmission(true);
					MovableMan:AddParticle(self.Screen);
					
					self.ScreenActivated = true;
				end
			end
		end -- for
		
		-- Check boot status
		if not isinside then
			-- Reset all completion signs if we're not inside the terminal zone
			self:AddObjectivePoint("Download Security Codes", self.DownloadArea:GetCenterPoint() + Vector(0 , - 25), CF_PlayerTeam, GameActivity.ARROWDOWN);
			if MovableMan:IsParticle(self.Screen) then
				MovableMan:RemoveParticle(self.Screen);
				self.Screen = nil;
			end
			
			self.ChunksComplete = 0;
			self.ScreenActivated = false;
			self.GenericTimer:Reset();
		else
			self:AddObjectivePoint(string.format("Downloading: %d%%", self.ChunksComplete / self.TotalChunks * 100), self.DownloadArea:GetCenterPoint() + Vector(0 , - 25), CF_PlayerTeam, GameActivity.ARROWDOWN);

			if self.GenericTimer:IsPastSimMS(self.ChunkInterval) then
				self.ChunksComplete = self.ChunksComplete + 1;

				if self.ChunksComplete == self.TotalChunks then
					print ("Data downloaded");
					MovableMan:RemoveParticle(self.Screen);
					self.Screen = nil;
					self.Screen = CreateAEmitter("Att01 Download Screen Done");
					self.Screen.Pos = Vector(192 , 264);
					self.Screen:EnableEmission(true);
					MovableMan:AddParticle(self.Screen);
					self:AdvanceToEvacuate();
				end

				self.GenericTimer:Reset();
			end
		end -- if - else
	end
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function Attack01:AdvanceToEvacuate()
	self.CurrentFightPhase = self.FightPhase.EVACUATE;
	self.DropshipSpawnInterval = self.DropshipSpawnInterval * 0.7;
end
-----------------------------------------------------------------------------------------
-- Phase NOFIGHT
-----------------------------------------------------------------------------------------
function Attack01:PhaseEvacuate()
	for player = 0, self.PlayerCount - 1 do
		if MovableMan:IsActor(self.AssignedBrains[player]) then
			self:AddObjectivePoint("EVACUATE!", self.AssignedBrains[player].AboveHUDPos, CF_PlayerTeam, GameActivity.ARROWUP);
		end
	end
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function Attack01:UpdateActivity()
   	self:ClearObjectivePoints();
	
	if not self.GenericTimer:IsPastSimMS(1500) then
		for player = 0, self.PlayerCount - 1 do
			self:ResetMessageTimer(player);
			FrameMan:ClearScreenText(player);
			FrameMan:SetScreenText("INFILTRATE ENEMY COMMUNICATION NODE!", player, 500, 10000, true);
		end
	end
	
	self:UpdateBaseDefenceStats();
	self:DistributeReinforcements();
	self:SpawnDropship();

	if self.CurrentFightPhase == self.FightPhase.NOFIGHT then
		self:PhaseNoFight();
	elseif self.CurrentFightPhase == self.FightPhase.DOWNLOAD then
		self:PhaseDownload();
	elseif self.CurrentFightPhase == self.FightPhase.EVACUATE then
		self:PhaseEvacuate();
	end	
	
	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	CF_ReturnOnMissionEnd();
	CF_CheckForeignTroops();
	-- end of generic part
	---------------------------------------------------------------------
	
	self:DoBrainSelection();
	
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function Attack01:DoBrainSelection()
	if not (self.ActivityState == Activity.OVER) then
		for player = 0, self.PlayerCount - 1 do
			local team = self:GetTeamOfPlayer(player);
	
			local brain = self:GetPlayerBrain(player);
			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				self:SetPlayerBrain(nil, player);
				local newBrain = MovableMan:GetUnassignedBrain(team);
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self:SwitchToActor(newBrain, player, team);
				else
					self.braindead[player] = true;
					
					if not MovableMan:GetClosestBrainActor(team) then
						if CF_Activity.ActivityState ~= Activity.OVER then
							---------------------------------------------------------------------
							-- Mission framework generic part. Must present in every mission!
							CF_CommitDefeat(self.Config , self.ModuleName , self.ConfigName);
							-- end of generic part
							---------------------------------------------------------------------
							self.WinnerTeam = self:OtherTeam(team);
							ActivityMan:EndActivity();
						end
					end
				end
			else
				self:SetObservationTarget(brain.Pos, player)
			end -- 
		end --for
	end --if
end
-----------------------------------------------------------------------------------------
-- Enter orbit, check winning conditions
-----------------------------------------------------------------------------------------
function Attack01:CraftEnteredOrbit()
	print (CF_BrainPresets[CF_PlayerFaction]);
	if self.OrbitedCraft:HasObject(CF_BrainPresets[CF_PlayerFaction]) then
		if CF_Activity.ActivityState ~= Activity.OVER then
			if self.CurrentFightPhase == self.FightPhase.EVACUATE then
				---------------------------------------------------------------------
				-- Mission framework generic part. Must present in every mission!
				CF_CommitVictory(self.Config , self.ModuleName , self.ConfigName);
				-- end of generic part
				---------------------------------------------------------------------
				self.WinnerTeam = CF_PlayerTeam;
			else
				---------------------------------------------------------------------
				-- Mission framework generic part. Must present in every mission!
				CF_CommitRetreat(self.Config , self.ModuleName , self.ConfigName);
				-- end of generic part
				---------------------------------------------------------------------
				self.WinnerTeam = CF_CPUTeam;
			end
			ActivityMan:EndActivity();
		end
	end
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------



































