-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function Attack04:StartActivity()
	print("START! -- Attack04:StartActivity()!");

	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	-- self.ConfigName = "Attack04.dat";
	self.ConfigName = "current.dat";
	self.ModuleName = "UnmappedLands.rte";
	
	self.Config = CF_ReadConfigFile(self.ModuleName , self.ConfigName);
	CF_InitFactions();
	CF_InitMission(self.Config);
	-- end of generic part	
	---------------------------------------------------------------------

	self.EnemyLZ = SceneMan.Scene:GetArea("EnemyLZ");
	
	self.braindead = {};
	self.AssignedBrains = {};
	
	for player = 0, self.PlayerCount - 1 do
        if not self:GetPlayerBrain(player) then
			self.braindead[player] = false;
			local brain = CF_SpawnBrain(CF_PlayerTeam , nil , CF_PlayerFaction , Actor.AIMODE_SENTRY);
			-- brain.Pos = Vector(192 , 264); -- Debug
			brain.Pos = Vector(2200 + player * 30 , 390); -- Gameplay
			brain.AIMode = Actor.AIMODE_SENTRY;
			self.AssignedBrains[player] = brain;

            MovableMan:AddActor(brain);
            self:SetPlayerBrain(brain, player);
            SceneMan:SetScroll(self:GetPlayerBrain(player).Pos, player);
            self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player);
			self:SwitchToActor(brain, player, CF_PlayerTeam);
        end
    end

	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	CF_ReplaceAddedActors(self.Config);
	CF_ArrangeEliteActors(self.Config);
	-- end of generic part	
	---------------------------------------------------------------------
	
    self.FightPhase = { SNEAKIN = 0 , ALERT = 1 , DEFENCE = 2};
    self.CurrentFightPhase = self.FightPhase.SNEAKIN;
	
	self.GenericTimer = Timer();
	self.GenericTimer:Reset();

	self.AmbushResetTimer = Timer();
	self.AmbushResetTimer:Reset();
	
	self.DropshipSpawnInterval = 8000;
	self.DropshipSpawnTimer = Timer();
	self.DropshipSpawnTimer:Reset();


	-- Zones defining firing point checked for troops precence
	self.PatrolZones = {}; 
	self.PatrolWaypoints = {};

	for i = 1 , 16 do
		self.PatrolZones[i] = SceneMan.Scene:GetArea("PatrolZone"..i);
		self.PatrolWaypoints[i] = {SceneMan.Scene:GetArea("PatrolPoint"..i.."L"):GetCenterPoint(), SceneMan.Scene:GetArea("PatrolPoint"..i.."R"):GetCenterPoint()};
	end

	self.EnumEven = false;
	
	for actor in MovableMan.AddedActors do
		actor.AIMode = Actor.AIMODE_SENTRY;
	end	

	-- Alert screens
	self.ScreenCount = 3;

	self.ScreensPos = {};

	self.ScreensPos[1] = Vector(804, 456);
	self.ScreensPos[2] = Vector(1212, 456);
	self.ScreensPos[3] = Vector(1008, 660);
	
	self.Screens = {};

	for i = 1, self.ScreenCount do
		self.Screens[i] = CreateAEmitter("Secured Screen" , self.ModuleName);
		self.Screens[i].Pos = self.ScreensPos[i];
		self.Screens[i]:EnableEmission(true);
		MovableMan:AddParticle(self.Screens[i]);
	end
	
	-- Debug
	--[[for actor in MovableMan.AddedActors do
		if actor.Team == CF_CPUTeam then
			if string.find(actor.PresetName ,"Brain") == nil then
				actor:GibThis();
			end
		end
	end
	--]]--

	
	local aibrain = CF_SpawnBrain(CF_CPUTeam , nil , CF_PlayerFaction , Actor.AIMODE_SENTRY);
	-- aibrain.Pos = Vector(1750, 590); -- Debug
	aibrain.Pos = Vector(286, 1033); -- Gameplay
	aibrain.AIMode = Actor.AIMODE_SENTRY;

	MovableMan:AddActor(aibrain);
	
	
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
function Attack04:PauseActivity(pause)
    print("PAUSE! -- Attack04:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
function Attack04:EndActivity()
    print("END! -- Attack04:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function Attack04:IsEven(number)
	if math.floor(number / 8) % 2 == 0 then
		return true;
	else
		return false;
	end
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function Attack04:PatrolLogic()
	-- Resend all enemy troops to closes patrol areas if alert is off
	-- Scan all patrol zones. If we have AIMODE_SENTRY troops there,
	-- GOTO them to the most far patrol point
	for actor in MovableMan.Actors do
		if self:IsEven(actor.ID) == self.EnumEven then
			if actor.Team == CF_CPUTeam and actor.AIMode ~= Actor.AIMODE_GOTO and actor.AIMode ~= Actor.AIMODE_BRAINHUNT then
				for i = 1 , #self.PatrolZones do
					if self.PatrolZones[i]:IsInside(actor.Pos) then
						-- Calc distances to available patrol waypoint and send actor to the most far
						local dist1 = math.sqrt((actor.Pos.X - self.PatrolWaypoints[i][1].X) ^ 2 + (actor.Pos.Y - self.PatrolWaypoints[i][1].Y) ^ 2);
						local dist2 = math.sqrt((actor.Pos.X - self.PatrolWaypoints[i][2].X) ^ 2 + (actor.Pos.Y - self.PatrolWaypoints[i][2].Y) ^ 2);
						
						-- actor:FlashWhite(50);
						actor.AIMode = Actor.AIMODE_GOTO;
						actor:ClearAIWaypoints();
						actor:DrawWaypoints(true);
						if dist1 < dist2 then
							actor:AddAISceneWaypoint(self.PatrolWaypoints[i][2]);
						else
							actor:AddAISceneWaypoint(self.PatrolWaypoints[i][1]);
						end --if
						break;
					end -- if
				end -- for
			end -- if 
		end
	end -- for
end
-----------------------------------------------------------------------------------------
-- Spawn dropship. Returns true if spawn is successfull, false otherwise
-----------------------------------------------------------------------------------------
function Attack04:SpawnDropship()
	if self.DropshipSpawnTimer:IsPastSimMS(self.DropshipSpawnInterval) then
		if MovableMan:GetMOIDCount() < CF_MOIDLimit - 20 then
			local ship = nil;
			local actor = nil;
			local price = 0;
			local totalprice = 0;
			local p;
			local landpos;
			
			-- Select landing zone
			landpos = Vector(self.EnemyLZ:GetRandomPoint().X , -10);
			
			ship , price = CF_SpawnDropship(CF_CPUTeam , landpos , CF_CPUFaction);
			totalprice = totalprice + price;
			
			local aimode;
			
			aimode = Actor.AIMODE_BRAINHUNT;
		
			-- Spawn armor or regular troopers depending on turn number to increase difficulty
			if math.random() < 0.30 then
				-- Give single dreadnought as dummy armor or two assalut troopers with shields
				-- to any other faction
				if CF_CPUFaction == CF_Factions.DUMMY then
					actor , price = CF_SpawnArmor(CF_CPUTeam , nil , CF_CPUFaction , aimode);
					totalprice = totalprice + price;
					
					ship:AddInventoryItem(actor);
				else
					for i = 1, 2 do
						actor , price = CF_SpawnArmor(CF_CPUTeam , nil , CF_CPUFaction , aimode);
						actor:AddInventoryItem(CreateHDFirearm("Heavy Digger" , "Base.rte"));
						totalprice = totalprice + price;

						ship:AddInventoryItem(actor);
					end
				end
			else
				for i = 1, 2 do
					actor , price = CF_SpawnRandomInfantry(CF_CPUTeam , nil , CF_CPUFaction , aimode);
					actor:AddInventoryItem(CreateHDFirearm("Heavy Digger" , "Base.rte"));
					totalprice = totalprice + price;

					ship:AddInventoryItem(actor);
				end
			end

			MovableMan:AddActor(ship);
		end
		self.DropshipSpawnTimer:Reset();
	end
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function Attack04:UpdateActivity()
   	self:ClearObjectivePoints();
	

	if self.CurrentFightPhase == self.FightPhase.SNEAKIN then
		self:PhaseSneakIn();
	elseif self.CurrentFightPhase == self.FightPhase.ALERT then
		self:PhaseAlert();
	elseif self.CurrentFightPhase == self.FightPhase.DEFENCE then
		self:PhaseDefence();
	end
	
	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	CF_ReturnOnMissionEnd();
	CF_CheckForeignTroops();
	-- end of generic part
	---------------------------------------------------------------------
	
	self:DoBrainSelection();
	
	self.EnumEven = not self.EnumEven;
	
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function Attack04:PhaseSneakIn()
	if not self.GenericTimer:IsPastSimMS(500) then
		for player = 0, self.PlayerCount - 1 do
			self:ResetMessageTimer(player);
			FrameMan:ClearScreenText(player);
			FrameMan:SetScreenText("SILENTLY ASSASSINATE ENEMY COMMANDER", player, 500, 8000, true);
		end
	end

	if self.GenericTimer:IsPastSimMS(8500) and not self.GenericTimer:IsPastSimMS(9000)then
		for player = 0, self.PlayerCount - 1 do
			self:ResetMessageTimer(player);
			FrameMan:ClearScreenText(player);
			FrameMan:SetScreenText("DON'T KILL ENEMY SOLDIERS TO AVOID ALERT", player, 500, 8000, true);
		end
	end
	
	if not self.GenericTimer:IsPastSimMS(5000) then
		for player = 0, self.PlayerCount - 1 do
			if MovableMan:IsActor(self.AssignedBrains[player]) then
				self:AddObjectivePoint("Protect Commander", self.AssignedBrains[player].AboveHUDPos, CF_PlayerTeam, GameActivity.ARROWDOWN);
			end
		end
	end

	self:PatrolLogic();
	
	if self:GetTeamDeathCount(CF_CPUTeam) > 0 then
		self:AdvanceToAlert();
	end
	
	
	if self.GenericTimer:IsPastSimMS(500) then
		local aihascommander = false;

		for actor in MovableMan.Actors do
			if actor.Team == CF_CPUTeam and string.find(actor.PresetName ,"Brain") then
				aihascommander = true;
				self:AddObjectivePoint("Assassinate", actor.AboveHUDPos, CF_PlayerTeam, GameActivity.ARROWDOWN);
			end
		end
		
		if not aihascommander then
			self:AdvanceToDefence();
		end
	end
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function Attack04:AdvanceToAlert()
	for i = 1, self.ScreenCount do
		self.Screens[i]:EnableEmission(false);
		MovableMan:RemoveParticle(self.Screens[i]);
		self.Screens[i] = nil;
		-- print (self.Screens[i]);

		self.Screens[i] = CreateAEmitter("Alert Screen" , self.ModuleName);
		self.Screens[i].Pos = self.ScreensPos[i];
		self.Screens[i]:EnableEmission(true);
		MovableMan:AddParticle(self.Screens[i]);
	end

	for actor in MovableMan.Actors do
		if actor.Team == CF_CPUTeam and actor.AIMode == Actor.AIMODE_GOTO then
			actor.AIMode = Actor.AIMODE_BRAINHUNT;
		end
	end
	
	self.CurrentFightPhase = self.FightPhase.ALERT;
	
	self.DropshipSpawnTimer:Reset();
	self.GenericTimer:Reset();
	
	-- Give player a 25 sec delay to finish his job if he really kills the CPU brain
	self.DropshipSpawnInterval = 25000;
	
	for player = 0, self.PlayerCount - 1 do
		self:ResetMessageTimer(player);
		FrameMan:ClearScreenText(player);
		FrameMan:SetScreenText("ENEMY ALERTED! INCOMING DROPSHIPS!!!", player, 500, 8000, true);
	end
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function Attack04:PhaseAlert()
	if self.GenericTimer:IsPastSimMS(25000) then
		self.DropshipSpawnInterval = 8000; -- Let the bodies flooooow
	end

	local aihascommander = false;
	
	for actor in MovableMan.Actors do
		if actor.Team == CF_CPUTeam and string.find(actor.PresetName ,"Brain") then
			aihascommander = true;
			self:AddObjectivePoint("Assassinate", actor.AboveHUDPos, CF_PlayerTeam, GameActivity.ARROWDOWN);
		end
	end
	
	if not aihascommander then
		self:AdvanceToDefence();
	end
	
	self:SpawnDropship();
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function Attack04:AdvanceToDefence()
	self.CurrentFightPhase = self.FightPhase.DEFENCE;
	
	self.DropshipSpawnTimer:Reset();
	self.GenericTimer:Reset();
	
	self.DropshipSpawnInterval = 14000;

	for actor in MovableMan.Actors do
		if actor.Team == CF_CPUTeam and actor.AIMode == Actor.AIMODE_GOTO then
			actor.AIMode = Actor.AIMODE_BRAINHUNT;
		end
	end
	
	for player = 0, self.PlayerCount - 1 do
		self:ResetMessageTimer(player);
		FrameMan:ClearScreenText(player);
		FrameMan:SetScreenText("SURVIVE ENEMY COUNTERATTACK!!!", player, 500, 8000, true);
	end
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function Attack04:PhaseDefence()
	if self.GenericTimer:IsPastSimMS(300000) then
		if CF_Activity.ActivityState ~= Activity.OVER then
			---------------------------------------------------------------------
			-- Mission framework generic part. Must present in every mission!
			CF_CommitVictory(self.Config , self.ModuleName , self.ConfigName);
			-- end of generic part
			---------------------------------------------------------------------
			self.WinnerTeam = CF_PlayerTeam;
			ActivityMan:EndActivity();
		end	
	end
	
	self:SpawnDropship();
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function Attack04:DoBrainSelection()
	if not (self.ActivityState == Activity.OVER) then
		for player = 0, self.PlayerCount - 1 do
			local team = self:GetTeamOfPlayer(player);
	
			local brain = self:GetPlayerBrain(player);
			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				self:SetPlayerBrain(nil, player);
				local newBrain = MovableMan:GetUnassignedBrain(team);
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self:SwitchToActor(newBrain, player, team);
				else
					self.braindead[player] = true;
					
					if not MovableMan:GetClosestBrainActor(team) then
						if CF_Activity.ActivityState ~= Activity.OVER then
							---------------------------------------------------------------------
							-- Mission framework generic part. Must present in every mission!
							CF_CommitDefeat(self.Config , self.ModuleName , self.ConfigName);
							-- end of generic part
							---------------------------------------------------------------------
							self.WinnerTeam = self:OtherTeam(team);
							ActivityMan:EndActivity();
						end
					end
				end
			else
				self:SetObservationTarget(brain.Pos, player)
			end -- 
		end --for
	end --if
end
-----------------------------------------------------------------------------------------
-- Enter orbit, check winning conditions
-----------------------------------------------------------------------------------------
function Attack04:CraftEnteredOrbit()
	print (CF_BrainPresets[CF_PlayerFaction]);
	if self.OrbitedCraft:HasObject(CF_BrainPresets[CF_PlayerFaction]) then
		if CF_Activity.ActivityState ~= Activity.OVER then
			---------------------------------------------------------------------
			-- Mission framework generic part. Must present in every mission!
			CF_CommitRetreat(self.Config , self.ModuleName , self.ConfigName);
			-- end of generic part
			---------------------------------------------------------------------
			self.WinnerTeam = CF_CPUTeam;
			ActivityMan:EndActivity();
		end
	end
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------







