-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function Defence01:StartActivity()
	print("START! -- Defence01:StartActivity()!");

	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	-- self.ConfigName = "Defence01.dat";
	self.ConfigName = "current.dat";
	self.ModuleName = "UnmappedLands.rte";
	
	self.Config = CF_ReadConfigFile(self.ModuleName , self.ConfigName);
	CF_InitFactions();
	CF_InitMission(self.Config);
	-- end of generic part	
	---------------------------------------------------------------------

	-- Variables to check how many dropships player killed in a row
	self.DropshipsDeployedInRow = 0;
	self.DropshipsToKillToWin = 2; -- debug
	-- self.DropshipsToKillToWin = 8; -- gameplay
	
	self.AreaAILandLZ = SceneMan.Scene:GetArea("AILandLZ");
	self.AreaAIBaseLZ = SceneMan.Scene:GetArea("AIBaseLZ");
	
	self.braindead = {};
	self.AssignedBrains = {};
	
	for player = 0, self.PlayerCount - 1 do
        if not self:GetPlayerBrain(player) then
			self.braindead[player] = false;
			local brain = CF_SpawnBrain(CF_PlayerTeam , nil , CF_PlayerFaction , Actor.AIMODE_SENTRY);
			brain.Pos = Vector(1400 + player * 30 , 780);
			brain.AIMode = Actor.AIMODE_SENTRY;
			self.AssignedBrains[player] = brain;

            MovableMan:AddActor(brain);
            self:SetPlayerBrain(brain, player);
            SceneMan:SetScroll(self:GetPlayerBrain(player).Pos, player);
            self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player);
			self:SwitchToActor(brain, player, CF_PlayerTeam);
        end
    end

	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	CF_ReplaceAddedActors(self.Config);
	CF_ArrangeGarrisonActors(self.Config);
	CF_ArrangeEliteActors(self.Config);
	-- end of generic part	
	---------------------------------------------------------------------
	
	self.GenericTimer = Timer();
	self.GenericTimer:Reset();

	self.DropshipSpawnTimer = Timer();
	self.DropshipSpawnInterval = 14000 - tonumber(self.Config["TurnNumber"]) * 250;
	if self.DropshipSpawnInterval < 8000 then
		self.DropshipSpawnInterval = 8000;
	end
	self.DropshipSpawnTimer:Reset();
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
function Defence01:PauseActivity(pause)
    print("PAUSE! -- Defence01:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
function Defence01:EndActivity()
    print("END! -- Defence01:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- Spawn dropship. Returns true if spawn is successfull, false otherwise
-----------------------------------------------------------------------------------------
function Defence01:SpawnDropship()
	if CF_AIBudget <= 0 then
		local enemyactors;
		
		enemyactors = 0;
		
		for actor in MovableMan.Actors do
			if actor.Team == CF_CPUTeam then
				enemyactors = enemyactors + 1;
				
				self:AddObjectivePoint("Kill", actor.AboveHUDPos, CF_PlayerTeam, GameActivity.ARROWDOWN);
			end
		end

		if CF_Activity.ActivityState ~= Activity.OVER then
			if enemyactors <= 1 then
				for player = 0, self.PlayerCount - 1 do
					self:ResetMessageTimer(player);
					FrameMan:ClearScreenText(player);
					FrameMan:SetScreenText("ENEMY RETREATED! VICTORY!!!", player, 1000, 8000, true);
				end

				---------------------------------------------------------------------
				-- Mission framework generic part. Must present in every mission!
				CF_CommitVictory(self.Config , self.ModuleName , self.ConfigName);
				-- end of generic part
				---------------------------------------------------------------------
				self.WinnerTeam = CF_PlayerTeam;
				ActivityMan:EndActivity();
			end
		end
	else
		if self.DropshipSpawnTimer:IsPastSimMS(self.DropshipSpawnInterval) then
			if MovableMan:GetMOIDCount() < CF_MOIDLimit - 20 then
				local ship = nil;
				local actor = nil;
				local price = 0;
				local totalprice = 0;
				local p;
				local dpos;

				
				if math.random() < 0.30 then
					dpos = Vector(self.AreaAIBaseLZ:GetRandomPoint().X , 0);
					p = 100;
				else
					dpos = Vector(self.AreaAILandLZ:GetRandomPoint().X , 0);

					p = 10 + tonumber(self.Config["TurnNumber"]);
					
					if p > 25 then 
						p = 25;
					end
				end
				
				ship , price = CF_SpawnDropship(CF_CPUTeam , dpos , CF_CPUFaction);
				totalprice = totalprice + price;
				
				
				-- Spawn armor or regular troopers depending on turn number to increase difficulty
				if p >= math.random(100) then
					-- Give single dreadnought as dummy armor or two assalut troopers with shields
					-- to any other faction
					if CF_CPUFaction == CF_Factions.DUMMY then
						actor , price = CF_SpawnArmor(CF_CPUTeam , nil , CF_CPUFaction , Actor.AIMODE_BRAINHUNT);
						totalprice = totalprice + price;

						ship:AddInventoryItem(actor);
					else
						for i = 1, 2 do
							actor , price = CF_SpawnArmor(CF_CPUTeam , nil , CF_CPUFaction , Actor.AIMODE_BRAINHUNT);
							totalprice = totalprice + price;

							ship:AddInventoryItem(actor);
						end
					end
				else
					for i = 1, 2 do
						actor , price = CF_SpawnRandomInfantry(CF_CPUTeam , nil , CF_CPUFaction , Actor.AIMODE_BRAINHUNT);
						totalprice = totalprice + price;

						ship:AddInventoryItem(actor);
					end
				end
				
				CF_AIBudget = CF_AIBudget - totalprice;
				print ("Spawn price "..totalprice);
				print ("AI budget "..CF_AIBudget);
				
				if CF_AIBudget < 0 then
					for player = 0, self.PlayerCount - 1 do
						self:ResetMessageTimer(player);
						FrameMan:ClearScreenText(player);
						FrameMan:SetScreenText("ENEMY RETREATING!\nELEMINATE REMAINING FORCES!", player, 500, 10000, true);
					end
				end
				
				self.DropshipsDeployedInRow = self.DropshipsDeployedInRow + 1;
				print("Dropship "..self.DropshipsDeployedInRow.." of "..self.DropshipsToKillToWin);
				
				-- if player killed self.DropshipsToKillToWin in a row then the enemy is retreating
				-- due to strong AA defence
				if self.DropshipsDeployedInRow >= self.DropshipsToKillToWin then
					print ("AI retreating due to heavy AA support");
					for player = 0, self.PlayerCount - 1 do
						self:ResetMessageTimer(player);
						FrameMan:ClearScreenText(player);
						FrameMan:SetScreenText("ENEMY RETREATING!\nELEMINATE REMAINING FORCES!", player, 500, 10000, true);
					end
					
					CF_AIBudget = 0;
				end

				MovableMan:AddActor(ship);
				self.DropshipSpawnTimer:Reset();
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function Defence01:UpdateActivity()
   	self:ClearObjectivePoints();
	
	if not self.GenericTimer:IsPastSimMS(1500) then
		for player = 0, self.PlayerCount - 1 do
			self:ResetMessageTimer(player);
			FrameMan:ClearScreenText(player);
			FrameMan:SetScreenText("ENEMY DROPSHIPS APPROACHING!\nDEFEND YOUR BASE FROM INCOMING FORCES!", player, 500, 10000, true);
		end
	end
	
	self:SpawnDropship();

	---------------------------------------------------------------------
	-- Mission framework generic part. Must present in every mission!
	CF_ReturnOnMissionEnd();
	CF_CheckForeignTroops();
	-- end of generic part
	---------------------------------------------------------------------
	
	self:DoBrainSelection();
	
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function Defence01:DoBrainSelection()
	if not (self.ActivityState == Activity.OVER) then
		for player = 0, self.PlayerCount - 1 do
			local team = self:GetTeamOfPlayer(player);
	
			local brain = self:GetPlayerBrain(player);
			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				self:SetPlayerBrain(nil, player);
				local newBrain = MovableMan:GetUnassignedBrain(team);
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self:SwitchToActor(newBrain, player, team);
				else
					self.braindead[player] = true;
					
					if not MovableMan:GetClosestBrainActor(team) then
						if CF_Activity.ActivityState ~= Activity.OVER then
							---------------------------------------------------------------------
							-- Mission framework generic part. Must present in every mission!
							CF_CommitDefeat(self.Config , self.ModuleName , self.ConfigName);
							-- end of generic part
							---------------------------------------------------------------------
							self.WinnerTeam = self:OtherTeam(team);
							ActivityMan:EndActivity();
						end
					end
				end
			else
				self:SetObservationTarget(brain.Pos, player)
			end -- 
		end --for
	end --if
end
-----------------------------------------------------------------------------------------
-- Enter orbit, check winning conditions
-----------------------------------------------------------------------------------------
function Defence01:CraftEnteredOrbit()
	if self.OrbitedCraft.Team == CF_PlayerTeam then
		if self.AssassinateComplete == true then
			--If a craft leaves with the brain robot...
			if self.OrbitedCraft:HasObject("Brain Robot") then
				if CF_Activity.ActivityState ~= Activity.OVER then
					-- Team 1 wins!
					self.EvacuateComplete = true;
					-- Tell the script to stop running for brains as the game has ended.
					for player = 0, self.PlayerCount - 1 do
						self.braindead[player] = true;
					end
					
					---------------------------------------------------------------------
					-- Mission framework generic part. Must present in every mission!
					CF_CommitVictory(self.Config , self.ModuleName , self.ConfigName);
					-- end of generic part
					---------------------------------------------------------------------
					self.WinnerTeam = CF_PlayerTeam;
					ActivityMan:EndActivity();
				end
			end
		else
			print (CF_BrainPresets[CF_PlayerFaction]);
			if self.OrbitedCraft:HasObject(CF_BrainPresets[CF_PlayerFaction]) then
				if CF_Activity.ActivityState ~= Activity.OVER then
					---------------------------------------------------------------------
					-- Mission framework generic part. Must present in every mission!
					CF_CommitRetreat(self.Config , self.ModuleName , self.ConfigName);
					-- end of generic part
					---------------------------------------------------------------------
					self.WinnerTeam = CF_CPUTeam;
					ActivityMan:EndActivity();
				end
			end
		end
	else
		if self.OrbitedCraft.Team == CF_CPUTeam then
			if CF_AIBudget > 0 then
				CF_AIBudget = CF_AIBudget + (self.OrbitedCraft.Health / 100 * CF_BasicDropshipPrice);
				print ("Dropship sold CF_AIBudget = " .. CF_AIBudget);
			end

			print("Dropship escaped. Dead in a row resetted");
			self.DropshipsDeployedInRow = 0;
		end
	end
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------



































